@@ -1,33 +1,39 @@
 """
+This sample demonstrates a desktop application that acquires a token interactively
+and then calls a web API with the token.
+
 Prerequisite is documented here:
 https://msal-python.readthedocs.io/en/latest/#msal.PublicClientApplication.acquire_token_interactive
 
-The configuration file would look like this:
+This sample loads its configuration from a .env file.
 
-{
-    "authority": "https://login.microsoftonline.com/organizations",
-    "client_id": "your_client_id",
-    "scope": ["User.ReadBasic.All"],
-        // You can find the other permission names from this document
-        // https://docs.microsoft.com/en-us/graph/permissions-reference
-    "username": "your_username@your_tenant.com",  // This is optional
-    "endpoint": "https://graph.microsoft.com/v1.0/users"
-        // You can find more Microsoft Graph API endpoints from Graph Explorer
-        // https://developer.microsoft.com/en-us/graph/graph-explorer
-}
+To make this sample work, you need to choose one of the following templates:
 
-You can then run this sample with a JSON configuration file:
+    .env.sample.entra-id
+    .env.sample.external-id
+    .env.sample.external-id-with-custom-domain
 
-    python sample.py parameters.json
+Copy the chosen template to a new file named .env, and fill in the values.
+
+You can then run this sample:
+
+    python name_of_this_script.py
 """
-import sys  # For simplicity, we'll read config file from 1st CLI param sys.argv[1]
-import json, logging, time, msal, requests
+import json
+import logging
+import os
+import time
+
+from dotenv import load_dotenv  # Need "pip install python-dotenv"
+import msal
+import requests
+
 
 # Optional logging
 # logging.basicConfig(level=logging.DEBUG)  # Enable DEBUG log for entire script
 # logging.getLogger("msal").setLevel(logging.INFO)  # Optionally disable MSAL DEBUG logs
 
-config = json.load(open(sys.argv[1]))
+load_dotenv()  # We use this to load configuration from a .env file
 
 # If for whatever reason you plan to recreate same ClientApplication periodically,
 # you shall create one global token cache and reuse it by each ClientApplication
@@ -36,12 +42,15 @@ global_token_cache = msal.TokenCache()  # The TokenCache() is in-memory.
 
 # Create a preferably long-lived app instance, to avoid the overhead of app creation
 global_app = msal.PublicClientApplication(
-    config["client_id"], authority=config["authority"],
+    os.getenv('CLIENT_ID'),
+    authority=os.getenv('AUTHORITY'),  # For Entra ID or External ID
+    oidc_authority=os.getenv('OIDC_AUTHORITY'),  # For External ID with custom domain
     #enable_broker_on_windows=True,  # Opted in. You will be guided to meet the prerequisites, if your app hasn't already
         # See also: https://docs.microsoft.com/en-us/azure/active-directory/develop/scenario-desktop-acquire-token-wam#wam-value-proposition
     token_cache=global_token_cache,  # Let this app (re)use an existing token cache.
         # If absent, ClientApplication will create its own empty token cache
     )
+scopes = os.getenv("SCOPE", "").split()
 
 
 def acquire_and_use_token():
@@ -49,7 +58,7 @@ def acquire_and_use_token():
     result = None
 
     # Firstly, check the cache to see if this end user has signed in before
-    accounts = global_app.get_accounts(username=config.get("username"))
+    accounts = global_app.get_accounts(username=os.getenv("USERNAME"))
     if accounts:
         logging.info("Account(s) exists in cache, probably with token too. Let's try.")
         print("Account(s) already signed in:")
@@ -58,15 +67,15 @@ def acquire_and_use_token():
         chosen = accounts[0]  # Assuming the end user chose this one to proceed
         print("Proceed with account: %s" % chosen["username"])
         # Now let's try to find a token in cache for this account
-        result = global_app.acquire_token_silent(config["scope"], account=chosen)
+        result = global_app.acquire_token_silent(scopes, account=chosen)
 
     if not result:
         logging.info("No suitable token exists in cache. Let's get a new one from AAD.")
         print("A local browser window will be open for you to sign in. CTRL+C to cancel.")
         result = global_app.acquire_token_interactive(  # Only works if your app is registered with redirect_uri as http://localhost
-            config["scope"],
+            scopes,
             #parent_window_handle=...,  # If broker is enabled, you will be guided to provide a window handle
-            login_hint=config.get("username"),  # Optional.
+            login_hint=os.getenv("USERNAME"),  # Optional.
                 # If you know the username ahead of time, this parameter can pre-fill
                 # the username (or email address) field of the sign-in page for the user,
                 # Often, apps use this parameter during reauthentication,
@@ -80,13 +89,17 @@ def acquire_and_use_token():
 
     if "access_token" in result:
         print("Token was obtained from:", result["token_source"])  # Since MSAL 1.25
-        # Calling graph using the access token
-        graph_response = requests.get(  # Use token to call downstream service
-            config["endpoint"],
-            headers={'Authorization': 'Bearer ' + result['access_token']},)
-        print("Graph API call result: %s ..." % graph_response.text[:100])
+        if os.getenv('ENDPOINT'):
+            # Calling a web API using the access token
+            api_result = requests.get(
+                os.getenv('ENDPOINT'),
+                headers={'Authorization': 'Bearer ' + result['access_token']},
+                ).json()  # Assuming the response is JSON
+            print("Web API call result", json.dumps(api_result, indent=2))
+        else:
+            print("Token acquisition result", json.dumps(result, indent=2))
     else:
-        print("Token acquisition failed")  # Examine result["error_description"] etc. to diagnose error
+        print("Token acquisition failed", result)  # Examine result["error_description"] etc. to diagnose error
 
 
 while True:  # Here we mimic a long-lived daemon
